@@ -42,6 +42,9 @@ from importlib import import_module
 from systemimage.bag import Bag
 
 
+LAST_UPDATE_FILE = '/userdata/.last_update'
+
+
 def safe_remove(path):
     """Like os.remove() but don't complain if the file doesn't exist."""
     try:
@@ -88,22 +91,40 @@ def _sortkey(item):
     return order.get(item[-1])
 
 
+class _Called:
+    # Defer importing named object until it's actually called.  This should
+    # reduce the instances of circular imports.
+    def __init__(self, path):
+        self._path, dot, self._name = path.rpartition('.')
+        if dot != '.':
+            raise ValueError
+
+    def _dig(self):
+        module = import_module(self._path)
+        return getattr(module, self._name)
+
+    def __call__(self, *args, **kws):
+        return self._dig()(*args, **kws)
+
+    def __eq__(self, other):
+        # Let class equality (and in-equality) work.
+        myself = self._dig()
+        return myself == other
+
+    def __ne__(self, other):
+        return not self.__eq__(other)
+
+
 def as_object(value):
     """Convert a Python dotted-path specification to an object.
 
     :param value: A dotted-path specification,
         e.g. the string `systemimage.scores.WeightedScorer`
+    :return: A proxy object that when called, performs the import and calls
+        the underyling object.
     :raises ValueError: when `value` is not dotted.
-    :raises ImportError: if the named module (i.e. up to the right-most dot)
-        does not exist.
-    :raises AttributeError: if the name after the right-most dot doesn't exist
-        in the named module.
     """
-    path, dot, name = value.rpartition('.')
-    if dot != '.':
-        raise ValueError
-    module = import_module(path)
-    return getattr(module, name)
+    return _Called(value)
 
 
 def as_timedelta(value):
@@ -186,15 +207,20 @@ def makedirs(dir):
 def last_update_date():
     """Return the last update date.
 
-    Taken from the mtime of /etc/system-image/channel.ini first, with fallback
-    to /etc/ubuntu-build if that file doesn't exist.
+    Taken from the mtime of the following files, in order:
+
+    - /userdata/.last_update
+    - /etc/system-image/channel.ini
+    - /etc/ubuntu-build
+
+    First existing path wins.
     """
     # Avoid circular imports.
     from systemimage.config import config
     channel_ini = os.path.join(
         os.path.dirname(config.config_file), 'channel.ini')
     ubuntu_build = config.system.build_file
-    for path in (channel_ini, ubuntu_build):
+    for path in (LAST_UPDATE_FILE, channel_ini, ubuntu_build):
         try:
             # Local time, since we can't know the timezone.
             timestamp = datetime.fromtimestamp(os.stat(path).st_mtime)
