@@ -30,7 +30,7 @@ import tarfile
 from contextlib import ExitStack
 from datetime import datetime, timezone
 from systemimage.config import config
-from systemimage.download import get_files
+from systemimage.download import DBusDownloadManager
 from systemimage.gpg import Context, SignatureError
 from systemimage.helpers import makedirs
 from urllib.parse import urljoin
@@ -93,8 +93,10 @@ def get_keyring(keyring_type, urls, sigkr, blacklist=None):
     ascxz_dst = tarxz_dst + '.asc'
     with ExitStack() as stack:
         # Let FileNotFoundError percolate up.
-        get_files([(tarxz_src, tarxz_dst),
-                   (ascxz_src, ascxz_dst)])
+        DBusDownloadManager().get_files([
+            (tarxz_src, tarxz_dst),
+            (ascxz_src, ascxz_dst),
+            ])
         stack.callback(os.remove, tarxz_dst)
         stack.callback(os.remove, ascxz_dst)
         signing_keyring = getattr(config.gpg, sigkr.replace('-', '_'))
